// -*- mode: c++ -*-
// $Id: Mesh.h,v 1.2 2004/01/18 22:30:36 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/Mesh.h,v $

#ifndef __MESH_H
#define __MESH_H

#include <set>
#include <vector>

#include "util.h"

class Triangle;
class Vertex;

class Mesh{
 public:
  // Typedefs for types
  typedef             std::vector<Vertex*>              VertexCt;
  typedef             std::vector<Triangle*>            TriangleCt;

  typedef             TriangleCt::iterator TriangleIt;
  typedef             VertexCt::iterator   VertexIt;
  typedef             VertexCt::reverse_iterator   VertexRIt;
  typedef             TriangleCt::const_iterator const_TriangleIt;
  typedef             VertexCt::const_iterator   const_VertexIt;
  typedef             VertexCt::const_reverse_iterator   const_VertexRIt;

  // Constructor / Destructor
  Mesh( void ) : m_vc(), m_tc() {}
  ~Mesh( void );

  // formatted i/o
  friend std::ostream& operator<<( std::ostream&, const Mesh& );
  friend std::istream& operator>>( std::istream&, Mesh& );

  // iterators
  TriangleIt          TriangleBegin( void ) { return m_tc.begin(); }
  TriangleIt          TriangleEnd( void ) { return m_tc.end(); }
  const_TriangleIt    TriangleBegin( void ) const { return m_tc.begin(); }
  const_TriangleIt    TriangleEnd( void ) const { return m_tc.end(); }
  size_t              TriangleSize( void ) const { return m_tc.size(); }
  VertexIt            VertexBegin( void ) { return m_vc.begin(); }
  VertexIt            VertexEnd( void ) { return m_vc.end(); }
  const_VertexIt      VertexBegin( void ) const { return m_vc.begin(); }
  const_VertexIt      VertexEnd( void ) const { return m_vc.end(); }
  size_t              VertexSize( void ) const { return m_vc.size(); }

  void addVertex( Vertex* v ) { m_vc.push_back( v ); }
  void addTriangle( Triangle* t ) { m_tc.push_back( t ); }

private:
  // all vertices
  VertexCt     m_vc;
  // all triangles
  TriangleCt   m_tc;

  // Construction functions
  void Init( void ) {
    // call functions to build topology
  }

  // add functions to build topology

  // it's not clear what the meaning of these would be...
  Mesh& operator=( const Mesh& ) { die(); return *this; }
};

#include <functional>

template <class T>
struct Delete : public std::unary_function<T*, void>{
  void operator()( T* x ) const { delete x; }
};

#endif /* __MESH_H */
