// -*- mode: c++ -*-
// $Id: Triangle.h,v 1.3 2004/01/26 18:03:29 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/Triangle.h,v $

#ifndef __TRIANGLE_H
#define __TRIANGLE_H

#include <iostream>
#include <vector>
#include <assert.h>

#include "util.h"
#include "Names.h"
#include "cvec.h"


class Vertex;

class Triangle{
 public:
  enum VertexLetter { A=0, B=1, C=2 };
  
  // Constructor / Destructor
  Triangle( Vertex*, Vertex*, Vertex* );
  ~Triangle( void );
  
  friend inline std::ostream& operator<<( std::ostream&, const Triangle& );
  friend inline std::istream& operator>>( std::istream&, Triangle& );

  // Accessors
  Vertex*             a( void ) const { return m_vs[A]; }
  Vertex*             b( void ) const { return m_vs[B]; }
  Vertex*             c( void ) const { return m_vs[C]; }
  Vertex* v( const VertexName vn ) const { assert( vn != VNONE ); return m_vs[vn]; }

  const CVec& N( void ) const { return m_normal; }
  CVec& N( void ) { return m_normal; }
  float Area( void ) const { return m_area; }

private:
  // Vertices ordered counter clockwise	
  Vertex*                 m_vs[3];
  
  // normal to the triangle
  CVec m_normal;
  float m_area;

  // think before you accidentally call these... (or think after as
  // the case may be)
  Triangle( const Triangle& ) { die(); }
  Triangle& operator=( const Triangle& ) { die(); return *this; }
};

#include "Vertex.h"

inline
Triangle::Triangle( Vertex* a, Vertex* b, Vertex* c )
  : m_normal( 0, 0, 0 ), m_area( 0 )
{
  assert( a && b && c );
  assert( ( a != b ) && ( b != c ) && ( c != a ) );
  // set the vertices
  m_vs[A] = a;  m_vs[B] = b;  m_vs[C] = c;

}

inline
Triangle::~Triangle( void ){
}

#include <iostream>

// do nothing at this time.
inline std::ostream& operator<<( std::ostream& os, const Triangle& ){ return os; }
inline std::istream& operator>>( std::istream& is, Triangle& ){ return is; }


#endif	/* __TRIANGLE_H */
