// -*- mode: c++ -*-
// $Id: Vertex.h,v 1.5 2004/01/31 03:19:19 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/Vertex.h,v $

#ifndef	__VERTEX_H__
#define	__VERTEX_H__

// include files
#include <iostream>
#include <iterator>

#include "cvec.h"
#include "Names.h"
#include "Triangle.h"

// class definition
class Vertex{
 public:
  inline Vertex( bool boundary = false );
  inline Vertex( const Vertex& v );
  ~Vertex( void ) {}

  friend inline std::ostream& operator<<( std::ostream&, const Vertex& );
  friend inline std::istream& operator>>( std::istream&, Vertex& );

  // position
  CVec& p( void ){ return m_p; }
  const CVec& p( void ) const{ return m_p; }
  // normal vector
  CVec& N( void ){ return m_normal; }
  const CVec& N( void ) const{ return m_normal; }


 protected:
 private:
  // point in space
  CVec m_p;
  // we are going to do flow so we'll need to update point
  // normal of the surface at that point
  CVec m_normal;

  // is this vertex on the boundary?
  bool m_boundary:1;
  
  Vertex& operator=( const Vertex& ) {
    die(); // why are you using assignment?
    return *this;
  }
};


// extern defs for non-member outline functions go here
// externally visible constant defs go here

inline std::ostream&
operator<<( std::ostream& os, const Vertex& v )
{
  return os << v.m_p;
}
  
inline std::istream&
operator>>( std::istream& is, Vertex& v )
{
  if( !( is >> v.m_p ) ) std::cerr << "Vertex: input failure\n";
  return is;
}
inline Vertex::Vertex( bool boundary )
  : m_p( 0, 0, 0 ), m_normal( 0, 0, 0 ),
    m_boundary( boundary )
{
}

inline Vertex::Vertex( const Vertex& v )
  : m_p( v.m_p ), m_normal( v.m_normal ), 
    m_boundary( v.m_boundary ) {}

#endif	/* __VERTEX_H__ */
