// -*- Mode: c++ -*-
// $Id: cvec.h,v 1.2 2004/01/31 02:43:40 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/cvec.h,v $
//

#ifndef	__CVEC_H__
#define	__CVEC_H__

// Do not change anything above this line

#include <iostream>
#include <stddef.h>
#include <math.h>
#include "util.h"

// column type 3 vectors
class CVec{
 public:
  enum{
    X = 0,
    Y = 1,
    Z = 2
    };
  CVec( void ) {}
  // PS: deprecated
  // CVec( const float f ) { m_v[X] = m_v[Y] = m_v[Z] = f; }
  CVec( const CVec& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; }
  CVec( const float a, const float b, const float c )
    { m_v[X] = a, m_v[Y] = b, m_v[Z] = c; }
  CVec( const float* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  CVec( const double* a )
    { m_v[X] = float( a[X] ), m_v[Y] = float( a[Y] ), m_v[Z] = float( a[Z] ); }
  operator float*( void )
    { return &m_v[X]; }
  operator const float*( void ) const
    { return &m_v[X]; }
  // ~CVec( void ) {}
  
  friend std::istream& operator>>( std::istream&, CVec& );
  friend std::ostream& operator<<( std::ostream&, const CVec& );
  void print( std::ostream& os ) const;
  bool read ( std::istream& is );
  
  // PS: not clear anymore what this was for...
  // bool operator==(const CVec& c)const
  // { return m_v[X]==c.m_v[X] && m_v[Y]==c.m_v[Y] && m_v[Z]==c.m_v[Z];}
  //bool operator<(const CVec& c)const 
  //  { return unsigned(this)<unsigned(&c); }
  
  CVec& operator=( const CVec& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; return *this; }
  CVec operator+( const CVec& c ) const
    { return CVec( m_v[X] + c.m_v[X], m_v[Y] + c.m_v[Y], m_v[Z] + c.m_v[Z] ); }
  CVec operator-( const CVec& c ) const
    { return CVec( m_v[X] - c.m_v[X], m_v[Y] - c.m_v[Y], m_v[Z] - c.m_v[Z] ); }
  CVec operator*( const float s ) const
    { return CVec( s * m_v[X], s * m_v[Y], s * m_v[Z] ); }
  friend CVec operator*( const float s, const CVec& c )
    { return CVec( s * c.m_v[X], s * c.m_v[Y], s * c.m_v[Z] ); }
  CVec operator/( const float s ) const
    { return CVec( m_v[X] / s, m_v[Y] / s, m_v[Z] / s ); }
  
  CVec& operator+=( const CVec& c )
    { m_v[X] += c.m_v[X], m_v[Y] += c.m_v[Y], m_v[Z] += c.m_v[Z]; return *this; }
  CVec& operator-=( const CVec& c )
    { m_v[X] -= c.m_v[X], m_v[Y] -= c.m_v[Y], m_v[Z] -= c.m_v[Z]; return *this; }
  CVec& operator*=( const float s )
    { m_v[X] *= s, m_v[Y] *= s, m_v[Z] *= s; return *this; }
  CVec& operator/=( const float s )
    { m_v[X] /= s, m_v[Y] /= s, m_v[Z] /= s; return *this; }
  CVec operator-( void ) const
    { return CVec( -m_v[X], -m_v[Y], -m_v[Z] ); }
  
  //PS: bogus
  //   CVec operator*(const CVec& c)const{
  //     return CVec(m_v[X]*c.m_v[X], m_v[Y]*c.m_v[Y], m_v[Z]*c.m_v[Z]);
  //   }
  //   CVec operator/(const CVec& c)const{
  //     return CVec(m_v[X]/c.m_v[X], m_v[Y]/c.m_v[Y], m_v[Z]/c.m_v[Z]);
  //   }
  
  float& x( void ) { return m_v[X]; }
  float x( void ) const { return m_v[X]; }
  float& y( void ) { return m_v[Y]; }
  float y( void ) const { return m_v[Y]; }
  float& z( void ) { return m_v[Z]; }
  float z( void ) const { return m_v[Z]; }
  float& operator() (const int i)      { return m_v[i]; }
  float operator() (const int i) const { return m_v[i]; }
  
  CVec lerp( const CVec& v1, const float t ) const{
    return CVec( m_v[X] + t * ( v1.m_v[X] - m_v[X] ),
		 m_v[Y] + t * ( v1.m_v[Y] - m_v[Y] ),
		 m_v[Z] + t * ( v1.m_v[Z] - m_v[Z] ) );
  }

  CVec min( const CVec& o ) const{
    const float a = std::min( m_v[X], o.m_v[X] );
    const float b = std::min( m_v[Y], o.m_v[Y] );
    const float c = std::min( m_v[Z], o.m_v[Z] );
    return CVec( a, b, c );
  }
  CVec max( const CVec& o ) const{
    const float a = std::max( m_v[X], o.m_v[X] );
    const float b = std::max( m_v[Y], o.m_v[Y] );
    const float c = std::max( m_v[Z], o.m_v[Z] );
    return CVec( a, b, c );
  }
  void bbox( CVec& min, CVec& max ){
    if( m_v[X] < min.m_v[X] ) min.m_v[X] = m_v[X];
    else if( m_v[X] > max.m_v[X] ) max.m_v[X] = m_v[X];
    if( m_v[Y] < min.m_v[Y] ) min.m_v[Y] = m_v[Y];
    else if( m_v[Y] > max.m_v[Y] ) max.m_v[Y] = m_v[Y];
    if( m_v[Z] < min.m_v[Z] ) min.m_v[Z] = m_v[Z];
    else if( m_v[Z] > max.m_v[Z] ) max.m_v[Z] = m_v[Z];
  }

  float dot( const CVec& c ) const
    { return m_v[X] * c.m_v[X] + m_v[Y] * c.m_v[Y] + m_v[Z] * c.m_v[Z]; }
  float dot( void ) const
    { return m_v[X] * m_v[X] + m_v[Y] * m_v[Y] + m_v[Z] * m_v[Z]; }
  CVec cross( const CVec& c ) const
    { return CVec( m_v[Y] * c.m_v[Z] - m_v[Z] * c.m_v[Y],
		  m_v[Z] * c.m_v[X] - m_v[X] * c.m_v[Z],
		  m_v[X] * c.m_v[Y] - m_v[Y] * c.m_v[X] ); }
  float l1( void ) const
    { float a = abs( m_v[X] ); a += abs( m_v[Y] ); return a + abs( m_v[Z] ); }
  float linfty( void ) const{
    float a = abs( m_v[X] ); a = std::max( a, abs( m_v[Y] ) );
    return std::max( a, abs( m_v[Z] ) );
  }
  float l2( void ) const { return sqrtf( dot() ); }
  
  CVec dir( void ) const;

  int normalize( void )
    { float mag = l2(); return zero( mag ) ? 0 : ( *this *= 1 / mag, 1 ); }
  float dist( const CVec& c ) const { return ( *this - c ).l2(); }
  // PS: enable if needed
  // int collinear( const CVec& a, const CVec& b ) const;
  // CVec perp( void ) const;
  
 protected:
 private:
  float	m_v[3];
};

typedef CVec* CVecp;

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#endif	/* __CVEC_H__ */
