// -*- Mode: c++ -*-
// $Id: main.cpp,v 1.7 2004/01/31 03:19:19 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/main.cpp,v $

// do not edit anything above this line

// include files
#include <iostream>
#include <sstream>
#include <assert.h>
#include <math.h>
#include <stdlib.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "uistate.h"
#include "Mesh.h"
#include "Vertex.h"
#include "Draw.h"
#include "MGeom.h"

// one global handle... Sorry; GLUT was written at a time when global
// handles weren't yet as violently discouraged. So we bundle
// everything up in UIState and then have a single entry point to it,
// which is hidden in this file...
static UIState *ui;
static Mesh *m;

void
Draw( void )
{
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  ui->SetupViewingTransform();

  // If you want to have the lights rotate with the object their
  // positions needs to be set AFTER the setup of the viewing
  // transform (above)
  // static GLfloat light1_position[] = { -.1f, .1f, 0, 0 };
  // glLightfv( GL_LIGHT1, GL_POSITION, light1_position );
  // static GLfloat light2_position[] = { .1f, .1f, 0, 0 };
  // glLightfv( GL_LIGHT2, GL_POSITION, light2_position );

  switch( ui->DStyle() ){
  case UIState::HIDDENLINE:
    DrawHiddenLine( m, ui->DirtyFlag() );
    break;
  case UIState::FLATSHADED:
    DrawFlatShaded( m, ui->DirtyFlag() );
    break;
  case UIState::SMOOTHSHADED:
    DrawSmoothShaded( m, ui->DirtyFlag() );
    break;
  default: std::cerr << "Draw: huh?\n";
  }
  // this draws the Ball controls; you may or may not like to see this...
  ui->Ball().Draw( ui->Aspect() );
  
  // show result
  glutSwapBuffers();
}

void
Reshape( const int width, const int height )
{
  if( width <= 0 || height <= 0 ) return;

  ui->WindowX() = width;
  ui->WindowY() = height;

  ui->Aspect() = float( width ) / height;
  ui->SetupViewport();
  ui->SetupViewingFrustum();
}

void
Visible( int state )
{
  // if visibility status of window changes
  switch( state ){
  case GLUT_VISIBLE:
    // install idle function if any
    glutIdleFunc( ui->Idle() );
    break;
  case GLUT_NOT_VISIBLE:
    glutIdleFunc( 0 );
    break;
  default:
    die();
  }
}

void
Keyboard( unsigned char c, int x, int y )
{
  // any non-standard ones can go here;
  switch( c ){
    // if you want to put in some key triggered thingie, this is the
    // place. You need to define the key you want and then take the
    // action. typically you'll then want to create a redraw even to
    // see the result of what you just did...
  case KEY_P:
    // do something
    std::cerr << "Hello world\n(going to redisplay now)\n";
    glutPostRedisplay();
    break;
  default:
    // if you didn't catch the key then maybe the standard code will
    // know what to do with the key
    ui->KeyEvent( c, x, y );
  }
}

void
SpecialKeys( int /*key*/, int /*x*/, int /*y*/ )
{
  // function keys and other non-alphanumeric keys
}

void
Motion( const int x, const int y )
{
  ui->Ball().Mouse( ( ui->Aspect() > 1 ? ui->Aspect() : 1 )
		    * ( 2.f * x / ui->WindowX() - 1 ),
		    -( ui->Aspect() > 1 ? 1 : 1/ui->Aspect() )
		    * ( 2.f * y / ui->WindowY() - 1 ) );
  if( ui->ButtonM() ){
    // translation case
    CVec t = ui->Ball().Trans();
    ui->CTrans().z() = ( 1 - 2 * t.y() ) * ui->COldTrans().z();
  }
  glutPostRedisplay();
}

void IgnoreMotion( const int, const int ) {}

void
Mouse( int button, int state, int x, int y )
{
  switch( button ){
  case GLUT_LEFT_BUTTON:
    if( state == GLUT_DOWN ) ui->OnLButtonDown( x, y );
    else if( state == GLUT_UP ) ui->OnLButtonUp( x, y );
    else die();
    break;
  case GLUT_MIDDLE_BUTTON:
    if( state == GLUT_DOWN ) ui->OnMButtonDown( x, y );
    else if( state == GLUT_UP ) ui->OnMButtonUp( x, y );
    else die();
    break;
#if 0
  case GLUT_RIGHT_BUTTON:
    if( state == GLUT_DOWN ) ui->OnRButtonDown( x, y );
    else if( state == GLUT_UP ) ui->OnRButtonUp( x, y );
    else die();
    break;
#endif	/* 0 */    
  default:
    // what else?
    die();
  }
}

void
MainMenu( int entry )
{
  // we set up the menu events to generate codes which are understood
  // by the Keyboard handler; this way everything is implemented only
  // once. You may need to do something more specific here if you want
  // something on a menu that does not have a simple keyboard
  // shortcut...
  Keyboard( entry, 0, 0 );
}

void
DrawMenu( int entry )
{
  ui->SetDstyle( UIState::DrawStyle( entry ) );
  ui->DirtyFlag() = 1;
  glutPostRedisplay();
}

void
GlutInit( void )
{
  // all the callbacks
  // for performance reasons we set those not used explicitly to 0
  glutDisplayFunc( Draw );
  glutReshapeFunc( Reshape );
  glutKeyboardFunc( Keyboard );
  glutSpecialFunc( SpecialKeys );
  glutMouseFunc( Mouse );
  glutMotionFunc( IgnoreMotion );
  glutPassiveMotionFunc( 0 );
  glutVisibilityFunc( Visible );
  glutMenuStatusFunc( 0 );
  glutEntryFunc( 0 );

  int dMenu = glutCreateMenu( DrawMenu );
  glutAddMenuEntry( "Hidden Line", UIState::HIDDENLINE );
  glutAddMenuEntry( "Flat Shaded", UIState::FLATSHADED );
  glutAddMenuEntry( "Smooth Shaded", UIState::SMOOTHSHADED );

  glutCreateMenu( MainMenu );
  glutAddSubMenu( "Draw", dMenu );
  glutAddMenuEntry( "Reset View [_]", KEY_SPACE );
  glutAddMenuEntry( "Hello World [p]", KEY_P );
  glutAddMenuEntry( "Exit [ESC]", KEY_ESCAPE );
  glutAttachMenu( GLUT_RIGHT_BUTTON );
}

void
GLInit( void )
{
  // setup gl stuff
  glClearColor( 1, 1, 1, 0 );
  glClearDepth( 1 );

  // white light at the eye
  GLfloat light0_position[] = { 0, 0, 1, 0 };
  GLfloat light0_diffuse[] = { 1, 1, 1, 1 };
  GLfloat light0_specular[] = { 1, 1, 1, 1 };
  glLightfv( GL_LIGHT0, GL_POSITION, light0_position );
  glLightfv( GL_LIGHT0, GL_DIFFUSE, light0_diffuse );
  glLightfv( GL_LIGHT0, GL_SPECULAR, light0_specular );

  //red light
  GLfloat light1_position[] = { -.1f, .1f, 0, 0 };
  GLfloat light1_diffuse[] = { .6f, .05f, .05f, 1 };
  GLfloat light1_specular[] = { .6f, .05f, .05f, 1 };
  glLightfv( GL_LIGHT1, GL_POSITION, light1_position );
  glLightfv( GL_LIGHT1, GL_DIFFUSE, light1_diffuse );
  glLightfv( GL_LIGHT1, GL_SPECULAR, light1_specular );

  //blue light
  GLfloat light2_position[] = { .1f, .1f, 0, 0 };
  GLfloat light2_diffuse[] = { .05f, .05f, .6f, 1 };
  GLfloat light2_specular[] = { .05f, .05f, .6f, 1 };
  glLightfv( GL_LIGHT2, GL_POSITION, light2_position );
  glLightfv( GL_LIGHT2, GL_DIFFUSE, light2_diffuse );
  glLightfv( GL_LIGHT2, GL_SPECULAR, light2_specular );
  
  //material
  GLfloat mat_ambient[] = { 0, 0, 0, 1 };
  GLfloat mat_specular[] = { .8f, .8f, .8f, 1 };
  GLfloat mat_diffuse[] = { .4f, .4f, .4f, 1 };
  GLfloat mat_shininess[] = { 128 };
  glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient );
  glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
  glMaterialfv( GL_FRONT, GL_DIFFUSE, mat_diffuse );
  glMaterialfv( GL_FRONT, GL_SHININESS, mat_shininess );

  GLfloat bmat_ambient[] = { 0, 0, 0, 1 };
  GLfloat bmat_specular[] = { 0, .8f, .8f, 1 };
  GLfloat bmat_diffuse[] = { 0, .4f, .4f, 1 };
  GLfloat bmat_shininess[] = { 128 };
  glMaterialfv( GL_BACK, GL_AMBIENT, bmat_ambient );
  glMaterialfv( GL_BACK, GL_SPECULAR, bmat_specular );
  glMaterialfv( GL_BACK, GL_DIFFUSE, bmat_diffuse );
  glMaterialfv( GL_BACK, GL_SHININESS, bmat_shininess );

  GLfloat lmodel_ambient[] = { 0, 0, 0, 1 };
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );

  // glEnable(GL_NORMALIZE);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT2);

  glDepthFunc( GL_LEQUAL);
  glEnable( GL_DEPTH_TEST );
  glDepthFunc(GL_LEQUAL);
  glPolygonOffset(1.0, 1.0);
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glDisable(GL_CULL_FACE);

  // if you want to add textures, this would be a place to do it
  // glEnable( GL_TEXTURE_2D );
  // glBindTexture( GL_TEXTURE_2D, m_map.GLName() );
}

void
AppInit( std::istream& is, int *, char *[] )
{
  m = new Mesh; assert( m );
  is >> *m;

  // most definitely we need to create the display list...
  ui->DirtyFlag() = 1;

  CVec l, u;
  ComputeBBox( m, l, u );

  // set the initial translation to the center of the bounding box;
  // this ensures that object local rotations are applied with respect
  // to this origin.
  ui->Trans() = .5f * ( l + u );

  // radius of the world
  ui->Radius() = .5f * l.dist( u );

  // setup correct near and far
  // contain the world between near and far
  ui->Near() = ui->Radius() / sinf( Deg2Rad( ui->Fov()/2 ) );
  ui->Far() = ui->Near() + 2 * ui->Radius();
  ui->CTrans().z() = -( ui->Near() + ui->Radius() );
  // give it some slop
  ui->Near() *= .1f;
  ui->Far() *= 10;
}

// good manners require that we have a usage message
const char *usage = "Simple Viewer:\n\tinput (obj file format triangle mesh) on stdin\nUI controls: right click for menu\narcball rotation interface with constraints (shift and ctrl)\n";

int
main( int argc, char *argv[] )
{
  if( argc > 1 ){ std::cerr << usage; exit(-1); }

  ui = new UIState; assert( ui );

  ui->Motion() = Motion;
  ui->IgnoreMotion() = IgnoreMotion;
  
  // let glut eat any command line args that it owns
  glutInit( &argc, argv );

  // full color, double buffered
  glutInitDisplayMode( GLUT_DEPTH | GLUT_RGBA | GLUT_DOUBLE );
  glutInitWindowSize( 512, 512 );
  glutCreateWindow( "SimpleViewer (ACM 106b; 2004)" );

  // setup remaining state
  GlutInit();
  GLInit();

  // your chance to do all the heavy lifting before
  // surrendering to the interface

  // we are going to read from stdin. Change if you want to read from
  // an explicit filename
  AppInit( std::cin, &argc, argv );

  // start processing events
  glutMainLoop();

  return 0;
}
