// $Id: matrix3x3.cpp,v 1.1.1.1 2004/01/18 02:59:31 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/matrix3x3.cpp,v $
//
//	matrix3x3.cpp
//
// all code definitions pertaining to the use of matrices
//
// $Log: matrix3x3.cpp,v $
// Revision 1.1.1.1  2004/01/18 02:59:31  ps
// Simple viewer framework for ACM106
//
// Revision 1.3  1998/09/03 18:23:47  ps
// ngediChanges spelling of StdAfx where applicable and made it include
// but turned it off inside itself with __sgi. The other way just didn't seem
// to want to work on the pC. Don't ask...

#include "matrix3x3.h"

Matrix3x3::Matrix3x3( const int i )
{
  if( i == 1 ){
    // for identity Matrix3x3
    mat[1] = mat[2] =
      mat[3] = mat[5] =
      mat[6] = mat[7] = 0.0;
    mat[0] = mat[4] = mat[8] = 1.0;
  }else if( i == 0 ){
    // for zero Matrix3x3
    mat[0] = mat[1] = mat[2] =
      mat[3] = mat[4] = mat[5] =
      mat[6] = mat[7] = mat[8] = 0.0;
  }else{
    std::cerr << "error in Matrix3x3::Matrix3x3( const int i ): i = " << i << "\n";
  }
}

Matrix3x3::Matrix3x3( const float x00, const float x01, const float x02,
		     const float x10, const float x11, const float x12,
		     const float x20, const float x21, const float x22 )
{
  mat[0] = x00; mat[1] = x01; mat[2] = x02;
  mat[3] = x10; mat[4] = x11; mat[5] = x12;
  mat[6] = x20; mat[7] = x21; mat[8] = x22;
}

Matrix3x3::Matrix3x3( const Matrix3x3& m )
{
  mat[0] = m.mat[0];
  mat[1] = m.mat[1];
  mat[2] = m.mat[2];
  mat[3] = m.mat[3];
  mat[4] = m.mat[4];
  mat[5] = m.mat[5];
  mat[6] = m.mat[6];
  mat[7] = m.mat[7];
  mat[8] = m.mat[8];
}

Matrix3x3&
Matrix3x3::operator=( const Matrix3x3& m )
{
  mat[0] = m.mat[0];
  mat[1] = m.mat[1];
  mat[2] = m.mat[2];
  mat[3] = m.mat[3];
  mat[4] = m.mat[4];
  mat[5] = m.mat[5];
  mat[6] = m.mat[6];
  mat[7] = m.mat[7];
  mat[8] = m.mat[8];
  
  return *this;
}

Matrix3x3
Matrix3x3::operator-( const Matrix3x3& m ) const
{
  return Matrix3x3(
    mat[0] - m.mat[0], mat[1] - m.mat[1], mat[2] - m.mat[2],
    mat[3] - m.mat[3], mat[4] - m.mat[4], mat[5] - m.mat[5],
    mat[6] - m.mat[6], mat[7] - m.mat[7], mat[8] - m.mat[8] );
}

Matrix3x3
Matrix3x3::operator+( const Matrix3x3& m ) const
{
  return Matrix3x3(
    mat[0] + m.mat[0], mat[1] + m.mat[1], mat[2] + m.mat[2],
    mat[3] + m.mat[3], mat[4] + m.mat[4], mat[5] + m.mat[5],
    mat[6] + m.mat[6], mat[7] + m.mat[7], mat[8] + m.mat[8] );
}

Matrix3x3
Matrix3x3::operator*( const Matrix3x3& m ) const
{
  return Matrix3x3(
    mat[0] * m.mat[0] + mat[1] * m.mat[3] + mat[2] * m.mat[6],
    mat[0] * m.mat[1] + mat[1] * m.mat[4] + mat[2] * m.mat[7],
    mat[0] * m.mat[2] + mat[1] * m.mat[5] + mat[2] * m.mat[8],
    mat[3] * m.mat[0] + mat[4] * m.mat[3] + mat[5] * m.mat[6],
    mat[3] * m.mat[1] + mat[4] * m.mat[4] + mat[5] * m.mat[7],
    mat[3] * m.mat[2] + mat[4] * m.mat[5] + mat[5] * m.mat[8],
    mat[6] * m.mat[0] + mat[7] * m.mat[3] + mat[8] * m.mat[6],
    mat[6] * m.mat[1] + mat[7] * m.mat[4] + mat[8] * m.mat[7],
    mat[6] * m.mat[2] + mat[7] * m.mat[5] + mat[8] * m.mat[8] );
}

Matrix3x3
Matrix3x3::operator*( const float s ) const
{
  return Matrix3x3(
    s * mat[0], s * mat[1], s * mat[2],
    s * mat[3], s * mat[4], s * mat[5],
    s * mat[6], s * mat[7], s * mat[8] );
}

Matrix3x3
Matrix3x3::operator/( const float s ) const
{
  // if you decide to divide by zero, that's your problem
  // however error checking would be very easy to add
  // if( fabs( s ) < epsilon )
  //	cerr << "Matrix3x3::operator/ division by zero\n";
  float div = 1.f / s;
  return Matrix3x3(
    div * mat[0], div * mat[1], div * mat[2],
    div * mat[3], div * mat[4], div * mat[5],
    div * mat[6], div * mat[7], div * mat[8] );
}

Matrix3x3
Matrix3x3::T() const
{
  return Matrix3x3(
    mat[0], mat[3], mat[6],
    mat[1], mat[4], mat[7],
    mat[2], mat[5], mat[8] );
}

Matrix3x3&
Matrix3x3::operator+=( const Matrix3x3& m )
{
  mat[0] += m.mat[0];
  mat[1] += m.mat[1];
  mat[2] += m.mat[2];
  mat[3] += m.mat[3];
  mat[4] += m.mat[4];
  mat[5] += m.mat[5];
  mat[6] += m.mat[6];
  mat[7] += m.mat[7];
  mat[8] += m.mat[8];
  
  return *this;
}

Matrix3x3&
Matrix3x3::operator-=( const Matrix3x3& m )
{
  mat[0] -= m.mat[0];
  mat[1] -= m.mat[1];
  mat[2] -= m.mat[2];
  mat[3] -= m.mat[3];
  mat[4] -= m.mat[4];
  mat[5] -= m.mat[5];
  mat[6] -= m.mat[6];
  mat[7] -= m.mat[7];
  mat[8] -= m.mat[8];
  
  return *this;
}

Matrix3x3&
Matrix3x3::operator*=( const float s )
{
  mat[0] *= s;
  mat[1] *= s;
  mat[2] *= s;
  mat[3] *= s;
  mat[4] *= s;
  mat[5] *= s;
  mat[6] *= s;
  mat[7] *= s;
  mat[8] *= s;
  
  return *this;
}

Matrix3x3&
Matrix3x3::operator*=( const Matrix3x3& m )
{
  float zero =
    mat[0] * m.mat[0] + mat[1] * m.mat[3] + mat[2] * m.mat[6];
  float one =
    mat[0] * m.mat[1] + mat[1] * m.mat[4] + mat[2] * m.mat[7];
  float two =
    mat[0] * m.mat[2] + mat[1] * m.mat[5] + mat[2] * m.mat[8];
  mat[0] = zero;
  mat[1] = one;
  mat[2] = two;
  zero =
    mat[3] * m.mat[0] + mat[4] * m.mat[3] + mat[5] * m.mat[6];
  one =
    mat[3] * m.mat[1] + mat[4] * m.mat[4] + mat[5] * m.mat[7];
  two =
    mat[3] * m.mat[2] + mat[4] * m.mat[5] + mat[5] * m.mat[8];
  mat[3] = zero;
  mat[4] = one;
  mat[5] = two;
  zero =
    mat[6] * m.mat[0] + mat[7] * m.mat[3] + mat[8] * m.mat[6];
  one =
    mat[6] * m.mat[1] + mat[7] * m.mat[4] + mat[8] * m.mat[7];
  two =
    mat[6] * m.mat[2] + mat[7] * m.mat[5] + mat[8] * m.mat[8];
  mat[6] = zero;
  mat[7] = one;
  mat[8] = two;
  
  return *this;
}

Matrix3x3&
Matrix3x3::operator/=( const float s )
{
  // if you decide to divide by zero, that's your problem
  // however error checking would be very easy to add
  // if( fabs( s ) < epsilon )
  //	cerr << "Matrix3x3::operator/= division by zero\n";
  float div = 1.f / s;
  mat[0] *= div;
  mat[1] *= div;
  mat[2] *= div;
  mat[3] *= div;
  mat[4] *= div;
  mat[5] *= div;
  mat[6] *= div;
  mat[7] *= div;
  mat[8] *= div;
  
  return *this;
}

Matrix3x3
Matrix3x3::operator-() const
{
  return Matrix3x3(
    -mat[0], -mat[1], -mat[2],
    -mat[3], -mat[4], -mat[5],
    -mat[6], -mat[7], -mat[8] );
}

std::ostream&
operator<<( std::ostream& out, const Matrix3x3& m )
{
  return out << "[ " << m.mat[0] << ", " << m.mat[1] << ", " << m.mat[2] <<
    " ]\n[ " << m.mat[3] << ", " << m.mat[4] << ", " << m.mat[5] <<
    " ]\n[ " << m.mat[6] << ", " << m.mat[7] << ", " << m.mat[8] << " ]";
}

#include "cvec.h"

CVec
Matrix3x3::operator*( const CVec& c ) const
{
  return CVec(
    mat[0] * c[0] + mat[1] * c[1] + mat[2] * c[2],
    mat[3] * c[0] + mat[4] * c[1] + mat[5] * c[2],
    mat[6] * c[0] + mat[7] * c[1] + mat[8] * c[2] );
}

CVec
Matrix3x3::T( const CVec& c ) const
{
  return CVec(
    mat[0] * c[0] + mat[3] * c[1] + mat[6] * c[2],
    mat[1] * c[0] + mat[4] * c[1] + mat[7] * c[2],
    mat[2] * c[0] + mat[5] * c[1] + mat[8] * c[2] );
}
