// -*- Mode: c++ -*-
// $Id: quaternion.cpp,v 1.1.1.1 2004/01/18 02:59:32 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/quaternion.cpp,v $
//
// Quaternion.C
//
// all functions for Quaternion algebra
//
// $Log: quaternion.cpp,v $
// Revision 1.1.1.1  2004/01/18 02:59:32  ps
// Simple viewer framework for ACM106
//
// Revision 1.2  1998/09/03 18:23:50  ps
// ngediChanges spelling of StdAfx where applicable and made it include
// but turned it off inside itself with __sgi. The other way just didn't seem
// to want to work on the pC. Don't ask...

#include "quaternion.h"

static	const	float	roundoff = 1.e-12f;

Quaternion
Quaternion::dir() const
{
  float l = len();
  if( l < roundoff )	l = 0;	else l = 1 / l;
  return Quaternion( s * l, CVec::operator*( l ) );
}

Quaternion
Quaternion::inv() const
{
  float d = dot( *this );
  if( d < roundoff )	d = 0;	else d = 1 / d;
  return Quaternion( s * d, CVec::operator*( -d ) );
}

Quaternion
Quaternion::operator/( const Quaternion& q ) const
{
  float d = q.dot( q );
  if( d < roundoff )	d = 0;	else d = 1 / d;
#if 0
  return Quaternion( d * ( s * q.s + CVec::dot( q ) ),
   	CVec::operator*( d * q.s ) - q.CVec::operator*( s * d ) -
   	CVec::cross( q ).operator*( d ) );
#endif /* 0 */
  return Quaternion(
    d * ( s * q.s +
    CVec(*this)[0] * CVec(q)[0] + CVec(*this)[1] * CVec(q)[1] + CVec(*this)[2] * CVec(q)[2] ),
    d * ( q.s * CVec(*this)[0] - s * CVec(q)[0] -
    CVec(*this)[1] * CVec(q)[2] + CVec(*this)[2] * CVec(q)[1] ),
    d * ( q.s * CVec(*this)[1] - s * CVec(q)[1] -
    CVec(*this)[2] * CVec(q)[0] + CVec(*this)[0] * CVec(q)[2] ),
    d * ( q.s * CVec(*this)[2] - s * CVec(q)[2] -
    CVec(*this)[0] * CVec(q)[1] + CVec(*this)[1] * CVec(q)[0] ) );
}

Quaternion
Quaternion::operator*( const Quaternion& q ) const
{
#if 0
  return Quaternion( s * q.s - CVec::dot( q ),
   	q.CVec::operator*( s ) + CVec::operator*( q.s ) +
  		CVec::cross( q ) );
#endif /* 0 */
  return Quaternion(
    s * q.s - CVec(*this)[0] * CVec(q)[0] - CVec(*this)[1] * CVec(q)[1] - CVec(*this)[2] * CVec(q)[2],
    s * CVec(q)[0] + q.s * CVec(*this)[0] + CVec(*this)[1] * CVec(q)[2] - CVec(*this)[2] * CVec(q)[1],
    s * CVec(q)[1] + q.s * CVec(*this)[1] + CVec(*this)[2] * CVec(q)[0] - CVec(*this)[0] * CVec(q)[2],
    s * CVec(q)[2] + q.s * CVec(*this)[2] + CVec(*this)[0] * CVec(q)[1] - CVec(*this)[1] * CVec(q)[0] );
}

std::ostream&
operator<<( std::ostream& out, const Quaternion& q )
{
  return out << "[ " << q.s << ", " << CVec( q ) << " ]";
}

CVec
Quaternion::rot( const CVec& c ) const
{
  float vxcx = CVec(*this)[1] * c[2] - CVec(*this)[2] * c[1];
  float vxcy = CVec(*this)[2] * c[0] - CVec(*this)[0] * c[2];
  float vxcz = CVec(*this)[0] * c[1] - CVec(*this)[1] * c[0];
  float fac = 2 * s;
  float rx = fac * vxcx - vxcy * CVec(*this)[2] + vxcz * CVec(*this)[1];
  float ry = fac * vxcy - vxcz * CVec(*this)[0] + vxcx * CVec(*this)[2];
  float rz = fac * vxcz - vxcx * CVec(*this)[1] + vxcy * CVec(*this)[0];
  fac = s * s;
  rx += fac * c[0];
  ry += fac * c[1];
  rz += fac * c[2];
  fac = CVec(*this)[0] * c[0] + CVec(*this)[1] * c[1] + CVec(*this)[2] * c[2];
  rx += fac * CVec(*this)[0];
  ry += fac * CVec(*this)[1];
  rz += fac * CVec(*this)[2];
  fac = s * s + CVec(*this)[0] * CVec(*this)[0] + CVec(*this)[1] * CVec(*this)[1] + CVec(*this)[2] * CVec(*this)[2];
  if( fac > roundoff ) fac = 1 / fac;
  return CVec( rx * fac, ry * fac, rz * fac );
}

Quaternion
Quaternion::operator*( const CVec& c ) const
{
  // return Quaternion( -CVec::T( c ),
  // 	c.CVec::operator*( s ) + CVec::cross( c ) );
  // spelled out
  return Quaternion(
    - CVec(*this)[0] * c[0] - CVec(*this)[1] * c[1] - CVec(*this)[2] * c[2],
    s * c[0] + CVec(*this)[1] * c[2] - CVec(*this)[2] * c[1],
    s * c[1] + CVec(*this)[2] * c[0] - CVec(*this)[0] * c[2],
    s * c[2] + CVec(*this)[0] * c[1] - CVec(*this)[1] * c[0] );
}

Quaternion
exp( const CVec& cv)
{
  float c = cv.l2(); float s = 1.;
  if( c > roundoff )	s = sinf( c ) / c;
  return Quaternion( cosf( c ), s * cv[0], s * cv[1], s * cv[2] );
}

CVec
log( const Quaternion& q )
{
  float sc = q.v().l2();
  float t = atan2f( sc, q.w() );
  if( sc > roundoff )	sc = t / sc;
  return CVec( sc * q.v() );
}
