// -*- Mode: c++ -*-
// $Id: quaternion.h,v 1.1.1.1 2004/01/18 02:59:32 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/quaternion.h,v $
//
// everything for orientations as quaternions
//
// $Log: quaternion.h,v $
// Revision 1.1.1.1  2004/01/18 02:59:32  ps
// Simple viewer framework for ACM106
//
// Revision 1.1.1.1  1998/09/01 19:34:27  jtr
// imported to multires repository

#ifndef	__QUATERNIONH
#define	__QUATERNIONH

#include <iostream>
#include <math.h>
#include "cvec.h"

class Matrix3x3;

class Quaternion : private CVec{
protected:
  float	s;	// actually cos( theta / 2 )
public:
  Quaternion() : CVec() {}
  Quaternion( const float w,
    const float xv, const float yv, const float zv )
    : CVec( xv, yv, zv ), s( w ) {}
  Quaternion( const float w, const CVec& v ) : CVec( v ), s( w ) {}
  Quaternion( const Quaternion& q ) : CVec( q ), s( q.s ) {}
  Quaternion( const Matrix3x3& );
  
  Quaternion&	operator=( const Quaternion& q )
		{ CVec::operator=( q ); s = q.s; return *this; }
  Quaternion&	operator=( const Matrix3x3& );
  
  friend	std::ostream&	operator<<( std::ostream&, const Quaternion& );
  
  Quaternion	operator*( const Quaternion& ) const;
  Quaternion	operator*( const CVec& ) const;
  Quaternion&	operator*=( const Quaternion& qt )
		{ return *this = *this * qt; }
  Quaternion&	operator*=( const CVec& qt )
		{ return *this = *this * qt; }
  Quaternion	operator*( const float sc ) const
		{ return Quaternion( s * sc, CVec::operator*( sc ) ); }
  friend	Quaternion	operator*( const float sc, const Quaternion& q )
		{ return Quaternion( q.s * sc, CVec( q ) * sc ); }
  Quaternion&	operator*=( const float sc )
		{ s *= sc; CVec::operator*=( sc ); return *this; }
  Quaternion	operator/( const float sc ) const
		{ return Quaternion( s / sc, CVec::operator/( sc ) ); }
  Quaternion	operator/( const Quaternion& ) const;
  Quaternion&	operator/=( const float sc )
		{ s /= sc; CVec::operator/=( sc ); return *this; }
  Quaternion	operator+( const Quaternion& qt ) const
		{ return Quaternion( s + qt.s, CVec::operator+( qt ) ); }
  Quaternion&	operator+=( const Quaternion& qt )
		{ s += qt.s; CVec::operator+=( qt ); return *this; }
  Quaternion	operator-( const Quaternion& qt ) const
		{ return Quaternion( s - qt.s, CVec::operator-( qt ) ); }
  Quaternion&	operator-=( const Quaternion& qt )
		{ s -= qt.s; CVec::operator-=( qt ); return *this; }
  Quaternion	dir() const;
  float	dot( const Quaternion& q ) const
		{ return s * q.s + CVec::dot( q ); }
  float	len() const	{ return float( sqrt( dot( *this ) ) ); }
  Quaternion	inv() const;
  CVec	rot( const CVec& ) const;
  Quaternion	T() const
  { return Quaternion( s, CVec::operator-() ); }

  float&	w()	{ return s; }
  const float&	w() const	{ return s; }
  CVec&	v()	{ return *this; }
  const CVec&	v() const	{ return *this; }
};

extern Quaternion exp( const CVec& );
extern CVec log( const Quaternion& );

#endif	/* __QUATERNIONH */
