// -*- Mode: c++ -*-
//
// quatmat.cpp
//
// all functions which refer to Matrix3x3 and Quaternion types

#include "quatmat.h"

// these codes are from SIGGRAPH course notes 23, Math for SIGGRAPH,
// 1989, page 203, 204

#include <math.h>

Quaternion::Quaternion( const Matrix3x3& m )
{
  double trace = m(0,0) + m(1,1) + m(2,2);
  if( fabs( trace ) > 1e-12f ){
    double w = sqrt( trace + 1 );
    s = float( w * .5f );
    w = .5 / w;
    (*this)[0] = float( ( m(2,1) - m(1,2) ) * w );
    (*this)[1] = float( ( m(0,2) - m(2,0) ) * w );
    (*this)[2] = float( ( m(1,0) - m(0,1) ) * w );
  }else{
    int i = 0;
    if( m(1,1) > m(0,0) )	i = 1;
    if( m(2,2) > m(i,i) )	i = 2;
    int j = ( i + 1 ) % 3;
    int k = ( j + 1 ) % 3;
    double w = sqrt( ( m(i,i) - ( m(j,j) + m(k,k) ) ) + 1.0 );
    (*this)[i] = float( w * .5 );
    w = .5 / w;
    s = float( ( m(k,j) - m(j,k) ) * w );
    (*this)[j] = float( ( m(j,i) + m(i,j) ) * w );
    (*this)[k] = float( ( m(k,i) + m(i,k) ) * w );
  }
}

Matrix3x3::Matrix3x3( const Quaternion& q )
{
  double w = 2.0 / q.dot( q );
  double xs = q.v()[0] * w; double ys = q.v()[1] * w;
  double zs = q.v()[2] * w; double ax = q.w() * xs;
  double ay = q.w() * ys; double az = q.w() * zs;
  double xx = q.v()[0] * xs; double xy = q.v()[0] * ys;
  double xz = q.v()[0] * zs; double yy = q.v()[1] * ys;
  double yz = q.v()[1] * zs; double zz = q.v()[2] * zs;
  
  mat[0] = float( 1.0 - ( yy + zz ) );
  mat[1] = float( xy - az );
  mat[2] = float( xz + ay );

  mat[3] = float( xy + az );
  mat[4] = float( 1.0 - ( xx + zz ) );
  mat[5] = float( yz - ax );
  
  mat[6] = float( xz - ay );
  mat[7] = float( yz + ax );
  mat[8] = float( 1.0 - ( xx + yy ) );
}
