// -*- Mode: c++ -*-
// $Id: uistate.h,v 1.2 2004/01/26 18:03:29 ps Exp $
// $Source: /cs/research/multires/cvsroot/src/frame/uistate.h,v $

#ifndef	__UISTATE_H__
#define	__UISTATE_H__

// Do not change anything above this line

// include files
#include <assert.h>
#include "ArcBall.h"
#include "cvec.h"

typedef void (*vpfv)( void );
typedef void (*vpfii)( const int x, const int y );

class UIState{
public:
  UIState( void );
  ~UIState( void ) {}

  // accessors
  int& MouseX( void ) { return p_mousex; }
  int& MouseY( void ) { return p_mousey; }
  int& WindowX( void ) { return p_windowx; }
  int& WindowY( void ) { return p_windowy; }
  float& Radius( void ) { return p_radius; }
  CVec& Trans( void ) { return p_trans; }
  CVec& CTrans( void ) { return p_ctrans; }
  CVec& COldTrans( void ) { return p_coldtrans; }
  ArcBall& Ball( void ) { return p_ball; }
  float& Aspect( void ) { return p_aspect; }
  float& Near( void ) { return p_near; }
  float& Far( void ) { return p_far; }
  float& Fov( void ) { return p_fov; }
  vpfv& Idle( void ) { return p_idle; }
  vpfii& Motion( void ) { return p_motion; }
  vpfii& IgnoreMotion( void ) { return p_ignoremotion; }
  enum{
    LEFT_BUTTON = 1,
    MIDDLE_BUTTON = 2,
    RIGHT_BUTTON = 4
  };
  int ButtonM( void ) { return p_button & MIDDLE_BUTTON; }

  // drawing styles
  enum DrawStyle{
    HIDDENLINE,
    FLATSHADED,
    SMOOTHSHADED,
  };
  DrawStyle DStyle( void ) { return p_dstyle; }
  void SetDstyle( DrawStyle s ) {
    assert( s == HIDDENLINE || s == FLATSHADED ||
	    s == SMOOTHSHADED);
    p_dstyle = s;
  }
  int& DirtyFlag( void ) { return p_dlistDirty; }

  // methods
  void ResetModelTransform( void );
  void SetupViewingTransform( void );
  void SetupViewingFrustum( void );
  void SetupViewport( void );

  void OnLButtonDown( const int x, const int y );
  void OnLButtonUp( const int x, const int y );
  void OnMButtonDown( const int x, const int y );
  void OnMButtonUp( const int x, const int y );
  void OnRButtonDown( const int x, const int y );
  void OnRButtonUp( const int x, const int y );
  void KeyEvent( const unsigned char c, const int x, const int y );

 protected:
 private:
  // ui state
  // last known mouse position
  int p_mousex, p_mousey;
  // last known window size
  int p_windowx, p_windowy;
  // which one is currently pressed?
  int p_button;
  // what style of drawing?
  DrawStyle p_dstyle;
  int p_dlistDirty;

  // radius of the world
  float p_radius;
  // translation due to centroid of object
  CVec p_trans;
  // camera translation to get a good view
  CVec p_ctrans;
  CVec p_coldtrans;

  // camera parameters
  float p_aspect;
  float p_near;
  float p_far;
  float p_fov;

  // function pointers for glut call backs
  vpfv p_idle;
  vpfii p_motion;
  vpfii p_ignoremotion;

  // arcball interface
  ArcBall p_ball;
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here
#include "util.h"

inline const float Deg2Rad( const float d ){
  return (float(M_PI)/180.f)*d;
}

// keys recognized
const int KEY_F = 'f';
const int KEY_P = 'p';
const int KEY_SPACE = ' ';
const int KEY_ESCAPE = '\033';

#endif	/* __UISTATE_H__ */
